@@ -237,7 +237,7 @@ _dbus_warn (const char *format,
     init_warnings ();
 
   if (fatal_warnings)
-    severity = DBUS_SYSTEM_LOG_FATAL;
+    severity = DBUS_SYSTEM_LOG_ERROR;
 
   va_start (args, format);
   _dbus_logv (severity, format, args);
@@ -269,7 +269,7 @@ _dbus_warn_check_failed(const char *format,
     init_warnings ();
 
   if (fatal_warnings_on_check_failed)
-    severity = DBUS_SYSTEM_LOG_FATAL;
+    severity = DBUS_SYSTEM_LOG_ERROR;
 
   va_start (args, format);
   _dbus_logv (severity, format, args);
@@ -395,7 +395,6 @@ _dbus_verbose_real (
   static dbus_bool_t need_pid = TRUE;
   int len;
   long sec, usec;
-  _dbus_get_real_time (&sec, &usec);
   
   /* things are written a bit oddly here so that
    * in the non-verbose case we just have the one
@@ -410,8 +409,9 @@ _dbus_verbose_real (
     {
       _dbus_print_thread ();
     }
-#endif
+  _dbus_get_real_time (&sec, &usec);
   fprintf (stderr, "%ld.%06ld ", sec, usec);
+#endif
 
   /* Only print pid again if the next line is a new line */
   len = strlen (format);
@@ -908,25 +908,8 @@ _dbus_get_local_machine_uuid_encoded (DBusString *uuid_str,
 
   if (machine_uuid_initialized_generation != _dbus_current_generation)
     {
-      DBusError local_error = DBUS_ERROR_INIT;
-
-      if (!_dbus_read_local_machine_uuid (&machine_uuid, FALSE,
-                                          &local_error))
-        {          
-#ifndef DBUS_ENABLE_EMBEDDED_TESTS
-          /* For the test suite, we may not be installed so just continue silently
-           * here. But in a production build, we want to be nice and loud about
-           * this.
-           */
-          _dbus_warn_check_failed ("D-Bus library appears to be incorrectly set up; failed to read machine uuid: %s\n"
-                                   "See the manual page for dbus-uuidgen to correct this issue.\n",
-                                   local_error.message);
-#endif
-          
-          dbus_error_free (&local_error);
-          
-          ok = _dbus_generate_uuid (&machine_uuid, error);
-        }
+      if (!_dbus_read_local_machine_uuid (&machine_uuid, FALSE, error))
+        ok = FALSE;
     }
 
   if (ok)
