@@ -40,6 +40,7 @@
 typedef struct {
     TestMainContext *ctx;
     DBusError e;
+    gboolean skip;
 
     DBusServer *server;
     DBusConnection *server_conn;
@@ -97,6 +98,14 @@ setup (Fixture *f,
   dbus_error_init (&f->e);
   g_queue_init (&f->server_messages);
 
+  if ((g_str_has_prefix (addr, "tcp:") ||
+       g_str_has_prefix (addr, "nonce-tcp:")) &&
+      !test_check_tcp_works ())
+    {
+      f->skip = TRUE;
+      return;
+    }
+
   f->server = dbus_server_listen (addr, &f->e);
   assert_no_error (&f->e);
   g_assert (f->server != NULL);
@@ -124,6 +133,9 @@ setup_runtime (Fixture *f,
 
   setup (f, addr);
 
+  if (f->skip)
+    return;
+
   listening_at = dbus_server_get_address (f->server);
   g_test_message ("listening at %s", listening_at);
   g_assert (g_str_has_prefix (listening_at, "unix:path="));
@@ -146,6 +158,9 @@ setup_no_runtime (Fixture *f,
 
   setup (f, addr);
 
+  if (f->skip)
+    return;
+
   listening_at = dbus_server_get_address (f->server);
   g_test_message ("listening at %s", listening_at);
   /* we have fallen back to something in /tmp, either abstract or not */
@@ -166,6 +181,9 @@ test_connect (Fixture *f,
   int n_entries;
   dbus_bool_t ok;
 
+  if (f->skip)
+    return;
+
   g_assert (f->server_conn == NULL);
 
   address = dbus_server_get_address (f->server);
@@ -266,13 +284,17 @@ test_bad_guid (Fixture *f,
     gconstpointer addr G_GNUC_UNUSED)
 {
   DBusMessage *incoming;
-  char *address = dbus_server_get_address (f->server);
+  char *address;
   gchar *guid;
 
+  if (f->skip)
+    return;
+
   g_test_bug ("39720");
 
   g_assert (f->server_conn == NULL);
 
+  address = dbus_server_get_address (f->server);
   g_assert (strstr (address, "guid=") != NULL);
   guid = strstr (address, "guid=");
   g_assert_cmpuint (strlen (guid), >=, 5 + 32);
@@ -328,6 +350,9 @@ test_message (Fixture *f,
   dbus_uint32_t serial;
   DBusMessage *outgoing, *incoming;
 
+  if (f->skip)
+    return;
+
   test_connect (f, addr);
 
   outgoing = dbus_message_new_signal ("/com/example/Hello",
@@ -439,6 +464,9 @@ main (int argc,
   g_test_add ("/message/nonce-tcp", Fixture, "nonce-tcp:host=127.0.0.1", setup,
       test_message, teardown);
 
+  g_test_add ("/message/bad-guid/tcp", Fixture, "tcp:host=127.0.0.1", setup,
+      test_bad_guid, teardown);
+
 #ifdef DBUS_UNIX
   g_test_add ("/connect/unix/tmpdir", Fixture, "unix:tmpdir=/tmp", setup,
       test_connect, teardown);
@@ -455,10 +483,10 @@ main (int argc,
   g_test_add ("/connect/unix/no-runtime", Fixture,
       "unix:runtime=yes;unix:tmpdir=/tmp", setup_no_runtime, test_connect,
       teardown_no_runtime);
-#endif
 
-  g_test_add ("/message/bad-guid", Fixture, "tcp:host=127.0.0.1", setup,
+  g_test_add ("/message/bad-guid/unix", Fixture, "unix:tmpdir=/tmp", setup,
       test_bad_guid, teardown);
+#endif
 
   return g_test_run ();
 }
